# ALMA Data Reduction Script

# Calibration

thesteps = []
step_title = {0: 'Import of the ASDM',
              1: 'Fix of SYSCAL table times',
              2: 'listobs',
              3: 'A priori flagging',
              4: 'Generation and time averaging of the WVR cal table',
              5: 'Generation of the Tsys cal table',
              6: 'Generation of the antenna position cal table',
              7: 'Application of the WVR, Tsys and antpos cal tables',
              8: 'Split out science SPWs and time average',
              9: 'Listobs, and save original flags',
              10: 'Initial flagging',
              11: 'Putting a model for the flux calibrator(s)',
              12: 'Save flags before bandpass cal',
              13: 'Bandpass calibration',
              14: 'Save flags before gain cal',
              15: 'Gain calibration',
              16: 'Save flags before applycal',
              17: 'Application of the bandpass and gain cal tables',
              18: 'Split out corrected column',
              19: 'Save flags after applycal'}

if 'applyonly' not in globals(): applyonly = True
try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

import os

if applyonly != True: es = aU.stuffForScienceDataReduction() 


if re.search('^4.7.0', casadef.casa_version) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 4.7.0')


# CALIBRATE_AMPLI: 
# CALIBRATE_ATMOSPHERE: J1733-1304,J1744-3116,J1924-2914,Sgr_B2N
# CALIBRATE_BANDPASS: J1924-2914
# CALIBRATE_FLUX: J1733-1304
# CALIBRATE_FOCUS: 
# CALIBRATE_PHASE: J1744-3116
# CALIBRATE_POINTING: J1733-1304,J1924-2914
# OBSERVE_CHECK: 
# OBSERVE_TARGET: Sgr_B2N

# Using reference antenna = DV12

# Import of the ASDM
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  if os.path.exists('uid___A002_Xb79187_X11db.ms') == False:
    importasdm('uid___A002_Xb79187_X11db', asis='Antenna Station Receiver Source CalAtmosphere CalWVR CorrelatorMode SBSummary', bdfflags=True, lazy=True, process_caldevice=False)
    if not os.path.exists('uid___A002_Xb79187_X11db.ms.flagversions'):
      print 'ERROR in importasdm. Output MS is probably not useful. Will stop here.'
      thesteps = []
  if applyonly != True: es.fixForCSV2555('uid___A002_Xb79187_X11db.ms')

# Fix of SYSCAL table times
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  from recipes.almahelpers import fixsyscaltimes
  fixsyscaltimes(vis = 'uid___A002_Xb79187_X11db.ms')

print "# A priori calibration"

# listobs
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb79187_X11db.ms.listobs')
  listobs(vis = 'uid___A002_Xb79187_X11db.ms',
    listfile = 'uid___A002_Xb79187_X11db.ms.listobs')
  
  

# A priori flagging
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagdata(vis = 'uid___A002_Xb79187_X11db.ms',
    mode = 'manual',
    spw = '5~12,17~32',
    autocorr = T,
    flagbackup = F)
  
  flagdata(vis = 'uid___A002_Xb79187_X11db.ms',
    mode = 'manual',
    intent = '*POINTING*,*ATMOSPHERE*',
    flagbackup = F)
  
  flagcmd(vis = 'uid___A002_Xb79187_X11db.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'plot',
    plotfile = 'uid___A002_Xb79187_X11db.ms.flagcmd.png')
  
  flagcmd(vis = 'uid___A002_Xb79187_X11db.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'apply')
  

# Generation and time averaging of the WVR cal table
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb79187_X11db.ms.wvr') 
  
  os.system('rm -rf uid___A002_Xb79187_X11db.ms.wvrgcal') 
  
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_Xb79187_X11db.ms.wvrgcal')
  
  wvrgcal(vis = 'uid___A002_Xb79187_X11db.ms',
    caltable = 'uid___A002_Xb79187_X11db.ms.wvr',
    spw = [25, 27, 29, 31],
    smooth = '6.048s',
    toffset = 0,
    tie = ['Sgr_B2N,J1744-3116'],
    statsource = 'Sgr_B2N')
  
  casalog.setlogfile(mylogfile)
  
  if applyonly != True: aU.plotWVRSolutions(caltable='uid___A002_Xb79187_X11db.ms.wvr', spw='25', antenna='DV12',
    yrange=[-199,199],subplot=22, interactive=False,
    figfile='uid___A002_Xb79187_X11db.ms.wvr.plots/uid___A002_Xb79187_X11db.ms.wvr') 
  
  #Note: If you see wraps in these plots, try changing yrange or unwrap=True 
  #Note: If all plots look strange, it may be a bad WVR on the reference antenna.
  #      To check, you can set antenna='' to show all baselines.
  

# Generation of the Tsys cal table
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb79187_X11db.ms.tsys') 
  gencal(vis = 'uid___A002_Xb79187_X11db.ms',
    caltable = 'uid___A002_Xb79187_X11db.ms.tsys',
    caltype = 'tsys')
  
  # Flagging edge channels
  
  flagdata(vis = 'uid___A002_Xb79187_X11db.ms.tsys',
    mode = 'manual',
    spw = '17:0~3;124~127,19:0~3;124~127,21:0~3;124~127,23:0~3;124~127',
    flagbackup = F)
  
  if applyonly != True: aU.plotbandpass(caltable='uid___A002_Xb79187_X11db.ms.tsys', overlay='time', 
    xaxis='freq', yaxis='amp', subplot=22, buildpdf=False, interactive=False,
    showatm=True,pwv='auto',chanrange='92.1875%',showfdm=True, showBasebandNumber=True, showimage=False, 
    field='', figfile='uid___A002_Xb79187_X11db.ms.tsys.plots.overlayTime/uid___A002_Xb79187_X11db.ms.tsys') 
  
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb79187_X11db.ms.tsys', msName='uid___A002_Xb79187_X11db.ms', interactive=False) 
  

# Generation of the antenna position cal table
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Position for antenna DV19 is derived from baseline run made on 2016-09-11 05:33:41.
  
  # Note: no baseline run found for antenna DV21.
  
  # Position for antenna DV12 is derived from baseline run made on 2016-09-11 05:33:41.
  
  # Note: no baseline run found for antenna DA49.
  
  # Position for antenna DA48 is derived from baseline run made on 2016-09-11 05:33:41.
  
  # Note: the correction for antenna DA45 is larger than 2mm.
  
  # Position for antenna DA45 is derived from baseline run made on 2016-09-11 05:33:41.
  
  # Note: no baseline run found for antenna DA58.
  
  # Position for antenna DV17 is derived from baseline run made on 2016-09-11 05:33:41.
  
  # Position for antenna DV01 is derived from baseline run made on 2016-09-11 05:33:41.
  
  # Position for antenna PM02 is derived from baseline run made on 2016-09-11 05:33:41.
  
  os.system('rm -rf uid___A002_Xb79187_X11db.ms.antpos') 
  gencal(vis = 'uid___A002_Xb79187_X11db.ms',
    caltable = 'uid___A002_Xb79187_X11db.ms.antpos',
    caltype = 'antpos',
    antenna = 'DA45,DA48,DV01,DV12,DV17,DV19,PM02',
  #  parameter = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
    parameter = [-8.22280e-04,6.39568e-04,2.28757e-03,3.97786e-04,-4.67816e-04,-1.34552e-04,-1.20464e-04,-1.86230e-04,1.48637e-05,-9.17286e-04,1.10415e-03,1.18926e-03,7.52620e-05,-4.16424e-04,-1.30291e-04,1.64712e-04,-2.75851e-04,-2.50258e-04,1.78201e-04,-7.97587e-04,-3.13033e-04])
  
  
  # antenna x_offset y_offset z_offset total_offset baseline_date
  # DA45    -8.22280e-04    6.39568e-04    2.28757e-03    2.51360e-03      2016-09-11 05:33:41
  # DV12    -9.17286e-04    1.10415e-03    1.18926e-03    1.86411e-03      2016-09-11 05:33:41
  # PM02     1.78201e-04   -7.97587e-04   -3.13033e-04    8.75151e-04      2016-09-11 05:33:41
  # DA48     3.97786e-04   -4.67816e-04   -1.34552e-04    6.28641e-04      2016-09-11 05:33:41
  # DV17     7.52620e-05   -4.16424e-04   -1.30291e-04    4.42774e-04      2016-09-11 05:33:41
  # DV19     1.64712e-04   -2.75851e-04   -2.50258e-04    4.07251e-04      2016-09-11 05:33:41
  # DV01    -1.20464e-04   -1.86230e-04    1.48637e-05    2.22293e-04      2016-09-11 05:33:41
  

# Application of the WVR, Tsys and antpos cal tables
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  
  from recipes.almahelpers import tsysspwmap
  tsysmap = tsysspwmap(vis = 'uid___A002_Xb79187_X11db.ms', tsystable = 'uid___A002_Xb79187_X11db.ms.tsys', tsysChanTol = 1)
  
  
  
  applycal(vis = 'uid___A002_Xb79187_X11db.ms',
    field = '0',
    spw = '25,27,29,31',
    gaintable = ['uid___A002_Xb79187_X11db.ms.tsys', 'uid___A002_Xb79187_X11db.ms.wvr', 'uid___A002_Xb79187_X11db.ms.antpos'],
    gainfield = ['0', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_Xb79187_X11db.ms',
    field = '1',
    spw = '25,27,29,31',
    gaintable = ['uid___A002_Xb79187_X11db.ms.tsys', 'uid___A002_Xb79187_X11db.ms.wvr', 'uid___A002_Xb79187_X11db.ms.antpos'],
    gainfield = ['1', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_Xb79187_X11db.ms',
    field = '2',
    spw = '25,27,29,31',
    gaintable = ['uid___A002_Xb79187_X11db.ms.tsys', 'uid___A002_Xb79187_X11db.ms.wvr', 'uid___A002_Xb79187_X11db.ms.antpos'],
    gainfield = ['2', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_Xb79187_X11db.ms',
    field = '3',
    spw = '25,27,29,31',
    gaintable = ['uid___A002_Xb79187_X11db.ms.tsys', 'uid___A002_Xb79187_X11db.ms.wvr', 'uid___A002_Xb79187_X11db.ms.antpos'],
    gainfield = ['3', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  if applyonly != True: es.getCalWeightStats('uid___A002_Xb79187_X11db.ms') 
  

# Split out science SPWs and time average
mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb79187_X11db.ms.split') 
  os.system('rm -rf uid___A002_Xb79187_X11db.ms.split.flagversions') 
  
  split(vis = 'uid___A002_Xb79187_X11db.ms',
    outputvis = 'uid___A002_Xb79187_X11db.ms.split',
    datacolumn = 'corrected',
    spw = '25,27,29,31',
    keepflags = T)
  
  

print "# Calibration"

# Listobs, and save original flags
mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb79187_X11db.ms.split.listobs')
  listobs(vis = 'uid___A002_Xb79187_X11db.ms.split',
    listfile = 'uid___A002_Xb79187_X11db.ms.split.listobs')
  
  
  if not os.path.exists('uid___A002_Xb79187_X11db.ms.split.flagversions/Original.flags'):
    flagmanager(vis = 'uid___A002_Xb79187_X11db.ms.split',
      mode = 'save',
      versionname = 'Original')
  
  

# Initial flagging
mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Flagging shadowed data
  
  flagdata(vis = 'uid___A002_Xb79187_X11db.ms.split',
    mode = 'shadow',
    flagbackup = F)
  
  

# Putting a model for the flux calibrator(s)
mystep = 11
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  setjy(vis = 'uid___A002_Xb79187_X11db.ms.split',
    standard = 'manual',
    field = 'J1733-1304',
    fluxdensity = [1.86359266556, 0, 0, 0],
    spix = -0.693637450434,
    reffreq = '202.105231308GHz')
  
  

# Save flags before bandpass cal
mystep = 12
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xb79187_X11db.ms.split',
    mode = 'save',
    versionname = 'BeforeBandpassCalibration')
  
  

# Bandpass calibration
mystep = 13
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb79187_X11db.ms.split.K_pre_bandpass') 
 
# Phase delay correction (found in spw2, DA45, DA48)
 
  gaincal(vis = 'uid___A002_Xb79187_X11db.ms.split',
    caltable = 'uid___A002_Xb79187_X11db.ms.split.K_pre_bandpass',
    field = '0', # J1924-2914
    spw = '0:0~1919,1:0~1919,2:0~1919,3:0~1919',
    scan = '1,3',
    solint = 'inf',
    gaintype = 'K',
    refant = 'DV12',
    calmode = 'p')

# Plot delay correction per spw

  os.system('rm -rf uid___A002_Xb79187_X11db.ms.K.plots') 
  os.system('mkdir uid___A002_Xb79187_X11db.ms.K.plots') 

  plotcal(caltable='uid___A002_Xb79187_X11db.ms.split.K_pre_bandpass',xaxis='freq',yaxis='delay',
          iteration='antenna',subplot=621,fontsize=7,
          figfile='uid___A002_Xb79187_X11db.ms.K.plots/uid___A002_Xb79187_X11db.ms.K.png')

  os.system('rm -rf uid___A002_Xb79187_X11db.ms.split.ap_pre_bandpass') 

  gaincal(vis = 'uid___A002_Xb79187_X11db.ms.split',
    caltable = 'uid___A002_Xb79187_X11db.ms.split.ap_pre_bandpass',
    field = '0', # J1924-2914
    spw = '0:0~1919,1:0~1919,2:0~1919,3:0~1919',
    scan = '1,3',
    solint = 'int',
    refant = 'DV12',
    gaintable = 'uid___A002_Xb79187_X11db.ms.split.K_pre_bandpass',
    calmode = 'p')
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb79187_X11db.ms.split.ap_pre_bandpass', msName='uid___A002_Xb79187_X11db.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_Xb79187_X11db.ms.split.bandpass') 
  bandpass(vis = 'uid___A002_Xb79187_X11db.ms.split',
    caltable = 'uid___A002_Xb79187_X11db.ms.split.bandpass',
    field = '0', # J1924-2914
    scan = '1,3',
    solint = 'inf',
    combine = 'scan',
    refant = 'DV12',
    solnorm = True,
    bandtype = 'B',
    gaintable = ['uid___A002_Xb79187_X11db.ms.split.ap_pre_bandpass','uid___A002_Xb79187_X11db.ms.split.K_pre_bandpass'])
  
  os.system('rm -rf uid___A002_Xb79187_X11db.ms.split.bandpass_smooth20ch') 
  
  bandpass(vis = 'uid___A002_Xb79187_X11db.ms.split',
    caltable = 'uid___A002_Xb79187_X11db.ms.split.bandpass_smooth20ch',
    field = '0', # J1924-2914
    scan = '1,3',
    solint = 'inf,8MHz',
    combine = 'scan',
    refant = 'DV12',
    solnorm = True,
    bandtype = 'B',
    gaintable = ['uid___A002_Xb79187_X11db.ms.split.ap_pre_bandpass','uid___A002_Xb79187_X11db.ms.split.K_pre_bandpass'])
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb79187_X11db.ms.split.bandpass_smooth20ch', msName='uid___A002_Xb79187_X11db.ms.split', interactive=False) 
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb79187_X11db.ms.split.bandpass', msName='uid___A002_Xb79187_X11db.ms.split', interactive=False) 
  

# Save flags before gain cal
mystep = 14
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xb79187_X11db.ms.split',
    mode = 'save',
    versionname = 'BeforeGainCalibration')
  
  

# Gain calibration
mystep = 15
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb79187_X11db.ms.split.phase_int') 
  gaincal(vis = 'uid___A002_Xb79187_X11db.ms.split',
    caltable = 'uid___A002_Xb79187_X11db.ms.split.phase_int',
    field = '0~2', # J1924-2914,J1733-1304,J1744-3116
    solint = 'int',
    refant = 'DV12',
    gaintype = 'G',
    calmode = 'p',
    gaintable = ['uid___A002_Xb79187_X11db.ms.split.bandpass_smooth20ch','uid___A002_Xb79187_X11db.ms.split.K_pre_bandpass'])
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb79187_X11db.ms.split.phase_int', msName='uid___A002_Xb79187_X11db.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_Xb79187_X11db.ms.split.ampli_inf') 
  gaincal(vis = 'uid___A002_Xb79187_X11db.ms.split',
    caltable = 'uid___A002_Xb79187_X11db.ms.split.ampli_inf',
    field = '0~2', # J1924-2914,J1733-1304,J1744-3116
    solint = 'inf',
    refant = 'DV12',
    gaintype = 'T',
    calmode = 'a',
    gaintable = ['uid___A002_Xb79187_X11db.ms.split.bandpass_smooth20ch', 'uid___A002_Xb79187_X11db.ms.split.phase_int','uid___A002_Xb79187_X11db.ms.split.K_pre_bandpass'])
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb79187_X11db.ms.split.ampli_inf', msName='uid___A002_Xb79187_X11db.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_Xb79187_X11db.ms.split.flux_inf') 
  os.system('rm -rf uid___A002_Xb79187_X11db.ms.split.fluxscale') 
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_Xb79187_X11db.ms.split.fluxscale')
  
  fluxscaleDict = fluxscale(vis = 'uid___A002_Xb79187_X11db.ms.split',
    caltable = 'uid___A002_Xb79187_X11db.ms.split.ampli_inf',
    fluxtable = 'uid___A002_Xb79187_X11db.ms.split.flux_inf',
    reference = '1') # J1733-1304
  
  casalog.setlogfile(mylogfile)
  
  if applyonly != True: es.fluxscale2(caltable = 'uid___A002_Xb79187_X11db.ms.split.ampli_inf', removeOutliers=True, msName='uid___A002_Xb79187_X11db.ms', writeToFile=True, preavg=10000)
  
  os.system('rm -rf uid___A002_Xb79187_X11db.ms.split.phase_inf') 
  gaincal(vis = 'uid___A002_Xb79187_X11db.ms.split',
    caltable = 'uid___A002_Xb79187_X11db.ms.split.phase_inf',
    field = '0~2', # J1924-2914,J1733-1304,J1744-3116
    solint = 'inf',
    refant = 'DV12',
    gaintype = 'G',
    calmode = 'p',
    gaintable = ['uid___A002_Xb79187_X11db.ms.split.bandpass_smooth20ch','uid___A002_Xb79187_X11db.ms.split.K_pre_bandpass'])
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb79187_X11db.ms.split.phase_inf', msName='uid___A002_Xb79187_X11db.ms.split', interactive=False) 
  

# Save flags before applycal
mystep = 16
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xb79187_X11db.ms.split',
    mode = 'save',
    versionname = 'BeforeApplycal')
  
  

# Application of the bandpass and gain cal tables
mystep = 17
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for i in ['0', '1']: # J1924-2914,J1733-1304
    applycal(vis = 'uid___A002_Xb79187_X11db.ms.split',
      field = str(i),
      gaintable = ['uid___A002_Xb79187_X11db.ms.split.K_pre_bandpass','uid___A002_Xb79187_X11db.ms.split.bandpass_smooth20ch', 'uid___A002_Xb79187_X11db.ms.split.phase_int', 'uid___A002_Xb79187_X11db.ms.split.flux_inf'],
      gainfield = ['','', i, i],
      interp = ['nearest','nearest','linear,linear','linear,linear'],
      calwt = T,
      flagbackup = F)
  
  applycal(vis = 'uid___A002_Xb79187_X11db.ms.split',
    field = '2,3', # Sgr_B2N
    gaintable = ['uid___A002_Xb79187_X11db.ms.split.K_pre_bandpass','uid___A002_Xb79187_X11db.ms.split.bandpass_smooth20ch', 'uid___A002_Xb79187_X11db.ms.split.phase_inf', 'uid___A002_Xb79187_X11db.ms.split.flux_inf'],
    gainfield = ['','', '2', '2'], # J1744-3116
    interp = ['nearest','nearest','linear,linear','linear,linear'],
    calwt = T,
    flagbackup = F)
  

# Split out corrected column
mystep = 18
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb79187_X11db.ms.split.cal') 
  os.system('rm -rf uid___A002_Xb79187_X11db.ms.split.cal.flagversions') 
  
  listOfIntents = ['CALIBRATE_BANDPASS#ON_SOURCE',
   'CALIBRATE_FLUX#ON_SOURCE',
   'CALIBRATE_PHASE#ON_SOURCE',
   'CALIBRATE_WVR#ON_SOURCE',
   'OBSERVE_TARGET#ON_SOURCE']
  
  split(vis = 'uid___A002_Xb79187_X11db.ms.split',
    outputvis = 'uid___A002_Xb79187_X11db.ms.split.cal',
    datacolumn = 'corrected',
    intent = ','.join(listOfIntents),
    keepflags = T)
  
  

# Save flags after applycal
mystep = 19
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xb79187_X11db.ms.split.cal',
    mode = 'save',
    versionname = 'AfterApplycal')
  
  

